@@ -7,15 +7,16 @@ from typing import List, Dict, Any, Tuple
 import requests
 from requests import Response
 
-from knowledge.base.entity import ThingObject, OntologyClassReference, LocalizedContent, OntologyPropertyReference, \
-    DataProperty, ObjectProperty, TenantAccessRight, LanguageCode, DATA_PROPERTIES_TAG, DATA_PROPERTY_TAG, \
+from knowledge.base.access import TenantAccessRight
+from knowledge.base.entity import LanguageCode, DATA_PROPERTIES_TAG, DATA_PROPERTY_TAG, \
     LANGUAGE_CODE_TAG, VALUE_TAG, IMAGE_TAG, DESCRIPTION_TAG, TYPE_TAG, URI_TAG, LABELS_TAG, IS_MAIN_TAG, \
-    DESCRIPTIONS_TAG, RELATIONS_TAG, EntityStatus, Label
+    DESCRIPTIONS_TAG, RELATIONS_TAG, EntityStatus, Label, Description
+from knowledge.base.ontology import DataProperty, OntologyPropertyReference, ThingObject, OntologyClassReference, \
+    ObjectProperty
 from knowledge.services.base import WacomServiceAPIClient, WacomServiceException
 
 # ------------------------------------------------- Constants ----------------------------------------------------------
 SEARCH_TERM: str = 'searchTerm'
-TEXT_TAG: str = 'text'
 LANGUAGE_PARAMETER: str = 'language'
 TYPES_PARAMETER: str = 'types'
 LIMIT_PARAMETER: str = 'limit'
@@ -125,18 +126,17 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         response: Response = requests.get(url, headers=headers, verify=self.verify_calls)
         if response.ok:
             e: Dict[str, Any] = response.json()
-            pref_label: List[LocalizedContent] = []
-            aliases: List[LocalizedContent] = []
+            pref_label: List[Label] = []
+            aliases: List[Label] = []
             # Extract labels and alias
             for label in e[LABELS_TAG]:
                 if label[IS_MAIN_TAG]:  # Labels
-                    pref_label.append(self.__import_locale__(label))
+                    pref_label.append(Label.create_from_dict(label))
                 else:  # Alias
-                    aliases.append(self.__import_locale__(label))
+                    aliases.append(Label.create_from_dict(label))
             # Create ThingObject
             thing: ThingObject = ThingObject(label=pref_label, icon=e[IMAGE_TAG],
-                                             description=[self.__import_locale__(d, DESCRIPTION_TAG)
-                                                          for d in e[DESCRIPTIONS_TAG]],
+                                             description=[Description.create_from_dict(d) for d in e[DESCRIPTIONS_TAG]],
                                              concept_type=OntologyClassReference.parse(e[TYPE_TAG]),
                                              uri=e[URI_TAG])
             # Set the alias
@@ -322,7 +322,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
             DATA_PROPERTIES_TAG: literals
         }
         if concept.tenant_access_right:
-            payload['tenantRights'] = concept.tenant_access_right.to_list()
+            payload[TENANT_RIGHTS_TAG] = concept.tenant_access_right.to_list()
         response: Response = requests.post(url, json=payload, headers=headers, verify=self.verify_calls)
         if response.ok:
             uri: str = response.json()[URI_TAG]
@@ -365,7 +365,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         raise WacomServiceException(f'Failed to pull relations. '
                                     f'Response code:={response.status_code}, exception:= {response.content}')
 
-    def labels(self, auth_key: str, uri: str) -> List[LocalizedContent]:
+    def labels(self, auth_key: str, uri: str) -> List[Label]:
         """
         Extract list labels of entity.
 
@@ -378,7 +378,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
 
         Returns
         -------
-        labels: List[LocalizedContent]
+        labels: List[Label]
             List of labels of an entity.
 
         Raises
@@ -394,7 +394,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         if response.ok:
             response_dict: dict = response.json()
             if LABELS_TAG in response_dict:
-                return [self.__import_locale__(label) for label in response_dict[LABELS_TAG]]
+                return [Label.create_from_dict(label) for label in response_dict[LABELS_TAG]]
             return []
         raise WacomServiceException('Failed to pull literals. Response code:={}, exception:= {}'
                                     .format(response.status_code, response.content))
@@ -412,8 +412,8 @@ class WacomKnowledgeService(WacomServiceAPIClient):
 
         Returns
         -------
-        labels: List[LocalizedContent]
-            List of literals of an entity.
+        labels: List[DataProperty]
+            List of data properties of an entity.
 
         Raises
         ------
@@ -568,7 +568,7 @@ class WacomKnowledgeService(WacomServiceAPIClient):
         limit: int
             Limit of the returned entities
         language_code: LanguageCode
-            Language code
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
 
         Returns
         -------
@@ -620,10 +620,19 @@ class WacomKnowledgeService(WacomServiceAPIClient):
     def ontology_update(self, auth_key: str):
         """
         Update the ontology.
+
+        **Remark:**
+        Works for users with role 'TenantAdmin'.
+
         Parameters
         ----------
         auth_key: str
+            Auth key from user
 
+        Raises
+        ------
+        WacomServiceException
+            If the graph service returns an error code and commit failed.
         """
         url: str = f'{self.service_base_url}{WacomKnowledgeService.ONTOLOGY_UPDATE_ENDPOINT}'
         # Header with auth token
@@ -635,10 +644,38 @@ class WacomKnowledgeService(WacomServiceAPIClient):
             raise WacomServiceException(f'Ontology update fails. '
                                         f'Response code:={response.status_code}, exception:= {response.content}')
 
-    def search_types(self, auth_key: str, search_term: str, language_code: LanguageCode,
-                     types: List[OntologyClassReference], limit: int = 30, next_page_id: str = None)\
+    def search_all(self, auth_key: str, search_term: str, language_code: LanguageCode,
+                   types: List[OntologyClassReference], limit: int = 30, next_page_id: str = None)\
             -> Tuple[List[ThingObject], str]:
+        """Search term in labels, literals and description.
 
+        Parameters
+        ----------
+        auth_key: str
+            Auth key from user
+        search_term: str
+            Search term.
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        types: List[OntologyClassReference]
+            Limits the types for search.
+        limit: int  (default:= 30)
+            Size of the page for pagination.
+        next_page_id: str (default:=None)
+            ID of the next page within pagination.
+
+        Returns
+        -------
+        results: List[ThingObject]
+            List of things matching the search term
+        next_page_id: str
+            ID of the next page.
+
+        Raises
+        ------
+        WacomServiceException
+            If the graph service returns an error code.
+        """
         headers: Dict[str, str] = {
             AUTHORIZATION_HEADER_FLAG:  f'Bearer {auth_key}'
         }
@@ -657,8 +694,34 @@ class WacomKnowledgeService(WacomServiceAPIClient):
                                     f'Response code:={response.status_code}, exception:= {response.content}')
 
     def search_labels(self, auth_key: str, search_term: str, language_code: LanguageCode, limit: int = 30,
-                      next_page_id: str = None)\
-            -> Tuple[List[ThingObject], str]:
+                      next_page_id: str = None) -> Tuple[List[ThingObject], str]:
+        """Search for matches in labels.
+
+        Parameters
+        ----------
+        auth_key: str
+            Auth key from user
+        search_term: str
+            Search term.
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        limit: int  (default:= 30)
+            Size of the page for pagination.
+        next_page_id: str (default:=None)
+            ID of the next page within pagination.
+
+        Returns
+        -------
+        results: List[ThingObject]
+            List of things matching the search term
+        next_page_id: str
+            ID of the next page.
+
+        Raises
+        ------
+        WacomServiceException
+            If the graph service returns an error code.
+        """
         headers: Dict[str, str] = {
             AUTHORIZATION_HEADER_FLAG:  f'Bearer {auth_key}'
         }
@@ -679,6 +742,38 @@ class WacomKnowledgeService(WacomServiceAPIClient):
                        pattern: SearchPattern = SearchPattern.REGEX,
                        language_code: LanguageCode = LanguageCode('en_US'),
                        limit: int = 30, next_page_id: str = None) -> Tuple[List[ThingObject], str]:
+        """
+        Search for matches in literals.
+
+        Parameters
+        ----------
+        auth_key: str
+            Auth key from user
+        search_term: str
+            Search term.
+        literal: OntologyPropertyReference
+            Literal used for the search
+        pattern: SearchPattern (default:= SearchPattern.REGEX)
+            Search pattern. The chosen search pattern must fit the type of the entity.
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        limit: int (default:= 30)
+            Size of the page for pagination.
+        next_page_id: str (default:=None)
+            ID of the next page within pagination.
+
+        Returns
+        -------
+        results: List[ThingObject]
+           List of things matching the search term
+       next_page_id: str
+           ID of the next page.
+
+       Raises
+       ------
+       WacomServiceException
+           If the graph service returns an error code.
+       """
         url: str = f'{self.service_url}/{self.service_endpoint}{WacomKnowledgeService.SEARCH_LITERALS_ENDPOINT}'
         parameters: Dict[str, Any] = {
             'Value': search_term,
@@ -699,8 +794,39 @@ class WacomKnowledgeService(WacomServiceAPIClient):
 
     def search_relation(self, auth_key: str, relation: OntologyPropertyReference,
                         language_code: LanguageCode, subject_uri: str = None, object_uri: str = None,
-                        limit: int = 30, next_page_id: str = None) \
-            -> Tuple[List[ThingObject], str]:
+                        limit: int = 30, next_page_id: str = None) -> Tuple[List[ThingObject], str]:
+        """
+        Search for matches in literals.
+
+        Parameters
+        ----------
+        auth_key: str
+            Auth key from user
+        relation: OntologyPropertyReference
+            Search term.
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        subject_uri: str (default:=None)
+            URI of the subject
+        object_uri: str (default:=None)
+            URI of the object
+        limit: int (default:= 30)
+            Size of the page for pagination.
+        next_page_id: str (default:=None)
+            ID of the next page within pagination.
+
+        Returns
+        -------
+        results: List[ThingObject]
+           List of things matching the search term
+        next_page_id: str
+           ID of the next page.
+
+       Raises
+       ------
+       WacomServiceException
+           If the graph service returns an error code.
+       """
         url: str = f'{self.service_url}/{self.service_endpoint}{WacomKnowledgeService.SEARCH_RELATION_ENDPOINT}'
         parameters: Dict[str, Any] = {
             SUBJECT_URI: subject_uri,
@@ -723,6 +849,33 @@ class WacomKnowledgeService(WacomServiceAPIClient):
 
     def search_description(self, auth_key: str, search_term: str, language_code: LanguageCode, limit: int = 30,
                            next_page_id: str = None) -> Tuple[List[ThingObject], str]:
+        """Search for matches in description.
+
+        Parameters
+        ----------
+        auth_key: str
+            Auth key from user
+        search_term: str
+            Search term.
+        language_code: LanguageCode
+            ISO-3166 Country Codes and ISO-639 Language Codes in the format '<language_code>_<country>, e.g., en_US.
+        limit: int  (default:= 30)
+            Size of the page for pagination.
+        next_page_id: str (default:=None)
+            ID of the next page within pagination.
+
+        Returns
+        -------
+        results: List[ThingObject]
+            List of things matching the search term
+        next_page_id: str
+            ID of the next page.
+
+        Raises
+        ------
+        WacomServiceException
+            If the graph service returns an error code.
+        """
         url: str = f'{self.service_base_url}{WacomKnowledgeService.SEARCH_DESCRIPTION_ENDPOINT}'
         parameters: Dict[str, Any] = {
             SEARCH_TERM: search_term,
@@ -747,16 +900,6 @@ class WacomKnowledgeService(WacomServiceAPIClient):
             results.append(ThingObject.from_dict(elem))
         return results, response[NEXT_PAGE_ID_TAG]
 
-    @staticmethod
-    def __import_locale__(param: Dict[str, Any], tag_name: str = 'value') -> LocalizedContent:
-        value: str = param[tag_name]
-        language_code: LanguageCode = param[LANGUAGE_CODE_TAG]
-        if IS_MAIN_TAG in param:
-            is_main: bool = bool(param[IS_MAIN_TAG])
-        else:
-            is_main: bool = True
-        return Label(content=value, language_code=language_code, main=is_main)
-
 
 class TenantManagementServiceAPI(WacomServiceAPIClient):
     """
