@@ -7,12 +7,27 @@ from typing import Dict, List, Any, Optional
 import requests
 from requests import Response
 
-from knowledge.base.entity import OntologyContext, OntologyProperty, LocalizedContent, PropertyType, OntologyClass, \
-    Label, Comment
-from knowledge.base.ontology import OntologyClassReference, OntologyPropertyReference
+from knowledge.base.entity import OntologyContext, LocalizedContent, Label, Comment
+from knowledge.base.ontology import OntologyClassReference, OntologyPropertyReference, OntologyProperty, OntologyClass,\
+    PropertyType
 from knowledge.services.base import WacomServiceAPIClient, WacomServiceException
 from knowledge.services.graph import AUTHORIZATION_HEADER_FLAG
 
+# ------------------------------------------------- Constants ----------------------------------------------------------
+BASE_URI_TAG: str = "baseUri"
+COMMENTS_TAG: str = "comments"
+DOMAIN_TAG: str = "domain"
+ICON_TAG: str = "icon"
+INVERSE_OF_TAG: str = "inverseOf"
+KIND_TAG: str = "kind"
+LABELS_TAG: str = "labels"
+LANGUAGE_CODE: str = 'locale'
+NAME_TAG: str = "name"
+RANGE_TAG: str = "range"
+SUB_CLASS_OF_TAG: str = "subClassOf"
+SUB_PROPERTY_OF_TAG: str = "subPropertyOf"
+TEXT_TAG: str = 'text'
+
 
 class OntologyService(WacomServiceAPIClient):
     """
@@ -35,6 +50,7 @@ class OntologyService(WacomServiceAPIClient):
     CONCEPT_ENDPOINT: str = 'context/{}/concepts/{}'
     PROPERTIES_ENDPOINT: str = "context/{}/properties"
     COMMIT_ENDPOINT: str = "context/{}/commit"
+    RDF_ENDPOINT: str = "context/{}/versions/rdf"
     PROPERTY_ENDPOINT: str = "context/{}/properties/{}"
 
     def __init__(self, service_url: str, service_endpoint: str = 'ontology'):
@@ -196,8 +212,7 @@ class OntologyService(WacomServiceAPIClient):
         context_url: str = urllib.parse.quote_plus(context_name)
         concept_url: str = urllib.parse.quote_plus(property_name)
         param: str = OntologyService.PROPERTY_ENDPOINT.format(context_url, concept_url)
-        response: Response = requests.get(f'{self.service_base_url}{param}',
-                                          headers=headers, verify=self.verify_calls)
+        response: Response = requests.get(f'{self.service_base_url}{param}', headers=headers, verify=self.verify_calls)
         if response.ok:
             return OntologyProperty.from_dict(response.json())
         raise WacomServiceException(f'Response code:={response.status_code}, exception:= {response.text}')
@@ -240,17 +255,17 @@ class OntologyService(WacomServiceAPIClient):
             AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
         payload: Dict[str, Any] = {
-            "subClassOf": subclass_of,
-            "name": name,
-            "labels": [],
-            "comments": [],
-            "icon": icon
+            SUB_CLASS_OF_TAG: subclass_of,
+            NAME_TAG: name,
+            LABELS_TAG: [],
+            COMMENTS_TAG: [],
+            ICON_TAG: icon
         }
         context_url: str = urllib.parse.quote_plus(context)
         for label in labels if labels is not None else []:
-            payload["labels"].append({'text': label.content, 'locale': label.language_code})
+            payload[LABELS_TAG].append({TEXT_TAG: label.content, LANGUAGE_CODE: label.language_code})
         for comment in comments if comments is not None else []:
-            payload["comments"].append({'text': comment.content, 'locale': comment.language_code})
+            payload[COMMENTS_TAG].append({TEXT_TAG: comment.content, LANGUAGE_CODE: comment.language_code})
         url: str = f'{self.service_base_url}{OntologyService.CONCEPTS_ENDPOINT.format(context_url)}'
         response: Response = requests.post(url, headers=headers, json=payload, verify=self.verify_calls)
         if response.ok:
@@ -261,7 +276,7 @@ class OntologyService(WacomServiceAPIClient):
 
     def update_concept(self, auth_key: str, context: str, name: str, subclass_of: Optional[str],
                        icon: Optional[str] = None, labels: Optional[List[Label]] = None,
-                       comments: Optional[List[Comment]] = None):
+                       comments: Optional[List[Comment]] = None) -> Dict[str, str]:
         """Update concept class.
 
         **Remark:**
@@ -284,6 +299,11 @@ class OntologyService(WacomServiceAPIClient):
         comments: Optional[List[Comment]] (default:= None)
             Comments for the class
 
+        Returns
+        -------
+        response: Dict[str, str]
+            Response from service
+
         Raises
         ------
         WacomServiceException
@@ -293,23 +313,22 @@ class OntologyService(WacomServiceAPIClient):
             AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
         payload: Dict[str, Any] = {
-            "subClassOf": subclass_of,
-            "name": name,
-            "labels": [],
-            "comments": [],
-            "icon": icon
+            SUB_CLASS_OF_TAG: subclass_of,
+            NAME_TAG: name,
+            LABELS_TAG: [],
+            COMMENTS_TAG: [],
+            ICON_TAG: icon
         }
         context_url: str = urllib.parse.quote_plus(context)
         for label in labels if labels is not None else []:
-            payload["labels"].append({'text': label.content, 'locale': label.language_code})
+            payload[LABELS_TAG].append({TEXT_TAG: label.content, LANGUAGE_CODE: label.language_code})
         for comment in comments if comments is not None else []:
-            payload["comments"].append({'text': comment.content, 'locale': comment.language_code})
+            payload[COMMENTS_TAG].append({TEXT_TAG: comment.content, LANGUAGE_CODE: comment.language_code})
         url: str = f'{self.service_url}{self.service_endpoint}' \
                    f'{OntologyService.CONCEPTS_ENDPOINT.format(context_url)}'
         response: Response = requests.put(url, headers=headers, json=payload, verify=self.verify_calls)
         if response.ok:
-            result_dict: Dict[str, str] = response.json()
-            return result_dict
+            return response.json()
         raise WacomServiceException(f'Update of concept failed. '
                                     f'Response code:={response.status_code}, exception:= {response.text}')
 
@@ -320,7 +339,10 @@ class OntologyService(WacomServiceAPIClient):
                         icon: Optional[str] = None,
                         labels: Optional[List[LocalizedContent]] = None,
                         comments: Optional[List[LocalizedContent]] = None) -> Dict[str, str]:
-        """Create concept class.
+        """Create property.
+
+        **Remark:**
+        Only works for users with role 'TenantAdmin'.
 
         Parameters
         ----------
@@ -346,6 +368,7 @@ class OntologyService(WacomServiceAPIClient):
             Labels for the class
         comments: Optional[List[Comment]] (default:= None)
             Comments for the class
+
         Returns
         -------
         result: Dict[str, str]
@@ -360,33 +383,34 @@ class OntologyService(WacomServiceAPIClient):
             AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
         payload: Dict[str, Any] = {
-            "kind": property_type.value,
-            "domain": domain_cls.iri,
-            "range": range_cls.iri,
-            "subPropertyOf": subproperty_of.iri,
-            "inverseOf": inverse_of.iri,
-            "name": name.iri,
-            "labels": [],
-            "comments": [],
-            "icon": icon
+            KIND_TAG: property_type.value,
+            DOMAIN_TAG: domain_cls.iri,
+            RANGE_TAG: range_cls.iri,
+            SUB_PROPERTY_OF_TAG: subproperty_of.iri,
+            INVERSE_OF_TAG: inverse_of.iri,
+            NAME_TAG: name.iri,
+            LABELS_TAG: [],
+            COMMENTS_TAG: [],
+            ICON_TAG: icon
         }
         context_url: str = urllib.parse.quote_plus(context)
         for label in labels if labels is not None else []:
-            payload["labels"].append({'text': label.content, 'locale': label.language_code})
+            payload[LABELS_TAG].append({TEXT_TAG: label.content, LANGUAGE_CODE: label.language_code})
         for comment in comments if comments is not None else []:
-            payload["comments"].append({'text': comment.content, 'locale': comment.language_code})
-        url: str = f'{self.service_url}{self.service_endpoint}' \
-                   f'{OntologyService.PROPERTIES_ENDPOINT.format(context_url)}'
+            payload[COMMENTS_TAG].append({TEXT_TAG: comment.content, LANGUAGE_CODE: comment.language_code})
+        url: str = f'{self.service_base_url}{OntologyService.PROPERTIES_ENDPOINT.format(context_url)}'
         response: Response = requests.post(url, headers=headers, json=payload, verify=self.verify_calls)
         if response.ok:
-            result_dict: Dict[str, str] = response.json()
-            return result_dict
+            return response.json()
         raise WacomServiceException(f'Update of concept failed. '
                                     f'Response code:={response.status_code}, exception:= {response.text}')
 
     def create_context(self, auth_key: str, name: str, base_uri: Optional[str] = None, icon: Optional[str] = None,
-                       labels: List[Label] = None, comments: List[Comment] = None) -> Dict[str, str] :
-        """Create context class.
+                       labels: List[Label] = None, comments: List[Comment] = None) -> Dict[str, str]:
+        """Create context.
+
+        **Remark:**
+        Only works for users with role 'TenantAdmin'.
 
         Parameters
         ----------
@@ -414,26 +438,24 @@ class OntologyService(WacomServiceAPIClient):
             If the ontology service returns an error code, exception is thrown.
         """
         headers: Dict[str, str] = {
-            'Authorization': f'Bearer {auth_key}'
+            AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
         payload: Dict[str, Any] = {
-            "baseUri": base_uri if base_uri is not None else f'wacom:{name}',
-            "name": name,
-            "labels": [],
-            "comments": [],
-            "icon": icon
+            BASE_URI_TAG: base_uri if base_uri is not None else f'wacom:{name}',
+            NAME_TAG: name,
+            LABELS_TAG: [],
+            COMMENTS_TAG: [],
+            ICON_TAG: icon
         }
         for label in labels if labels is not None else []:
-            payload["labels"].append({'text': label.content, 'locale': label.language_code})
+            payload[LABELS_TAG].append({TEXT_TAG: label.content, LANGUAGE_CODE: label.language_code})
         for comment in comments if comments is not None else []:
-            payload["comments"].append({'text': comment.content, 'locale': comment.language_code})
+            payload[COMMENTS_TAG].append({TEXT_TAG: comment.content, LANGUAGE_CODE: comment.language_code})
         url: str = f'{self.service_base_url}{OntologyService.CONTEXT_ENDPOINT}'
-
         response: Response = requests.post(url, headers=headers, json=payload, verify=self.verify_calls)
         if response.ok:
-            result_dict: Dict[str, str] = response.json()
-            return result_dict
-        raise WacomServiceException(f'Update of concept failed. '
+            return response.json()
+        raise WacomServiceException(f'Creation of concept failed. '
                                     f'Response code:={response.status_code}, exception:= {response.text}')
 
     def commit(self, auth_key: str, context_name: str):
@@ -448,7 +470,7 @@ class OntologyService(WacomServiceAPIClient):
             Name of the context.
         """
         headers: Dict[str, str] = {
-            'Authorization': f'Bearer {auth_key}'
+            AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
         }
         context_url: str = urllib.parse.quote_plus(context_name)
         url: str = f'{self.service_base_url}{OntologyService.COMMIT_ENDPOINT.format(context_url)}'
@@ -456,3 +478,30 @@ class OntologyService(WacomServiceAPIClient):
         if not response.ok:
             raise WacomServiceException(f'Commit of ontology failed. '
                                         f'Response code:={response.status_code}, exception:= {response.text}')
+
+    def rdf_export(self, auth_key: str, context_name: str) -> str:
+        """
+        Export RDF.
+
+        Parameters
+        ----------
+        auth_key: str
+            User token (must have TenantAdmin) role
+        context_name: str
+            Name of the context.
+
+        Returns
+        -------
+        rdf: str
+            Ontology as RDFS / OWL  ontology
+        """
+        headers: Dict[str, str] = {
+            AUTHORIZATION_HEADER_FLAG: f'Bearer {auth_key}'
+        }
+        context_url: str = urllib.parse.quote_plus(context_name)
+        url: str = f'{self.service_base_url}{OntologyService.RDF_ENDPOINT.format(context_url)}'
+        response: Response = requests.get(url, headers=headers, verify=self.verify_calls)
+        if response.ok:
+            return response.text
+        raise WacomServiceException(f'RDF export failed. '
+                                    f'Response code:={response.status_code}, exception:= {response.text}')
